/*
    Author: Nedyalko Prisadnikov
*/
// Eppur Si Muove
#include <iostream>
#include <vector>
#include <string>
#include <algorithm>
#include <cmath>
#include <ctime>
#include <cstdlib>
#include <cstdio>
#include <cstring>
#include <fstream>
#include <set>
#include <sys/time.h>
#include <ctime>
#include <bitset>

using namespace std;

// simple way to represent possible neighbours
int mx[] = {1, -1, 0, 0};
int my[] = {0, 0, 1, -1};

// input variables

#define MAXN 351
#define MAXC 501
int n, c;
int mat[MAXN][MAXN];

clock_t started;

// end of input variables

// variables to keep the best solution
int bestScore;
int bestSeedRow;
int bestSeedCol;
int bestColors[MAXN*MAXN];
// end of variables to keep the best solution

// variables to keep current solution
int score;
int seedRow;
int seedCol;
int colors[MAXN*MAXN];
// end of variables to keep current solution

// variables
int used[MAXN][MAXN];
int vertex[MAXN][MAXN];
int n_vertices;
set<int> adj[MAXN*MAXN];
int vertexRow[MAXN*MAXN];
int vertexCol[MAXN*MAXN];
int status[MAXN*MAXN];
int color[MAXN*MAXN];
bitset<MAXN*MAXN> value[MAXC];
int nValue[MAXC];
int nReady[MAXC];
int ready[MAXC][MAXN*MAXN];
int cc[MAXC];

int * A[MAXN*MAXN];
int nA[MAXN*MAXN];
// end of variables

int random_number(int minn, int maxx) {
    return minn + rand() % (maxx - minn + 1);
}

//double get_time() {
//    timeval tv;
//    gettimeofday(&tv, 0);
//    return tv.tv_sec + tv.tv_usec * 1e-6;
//}

// read the input from the standard input
void read_input() {
    ifstream fin("flood-it.in");
    fin >> n >> c;
    for (int row = 0; row < n; ++row) {
        for (int col = 0; col < n; ++col) {
            fin >> mat[row][col];
        }
    }
}

void output_best_solution() {
    ofstream fout("flood-it.out");
    fout << bestScore << endl;
    fout << bestSeedRow << " " << bestSeedCol << endl;
    for (int ind = 0; ind < bestScore; ++ind) {
        fout << bestColors[ind] << endl;
    }
}

void output_standard_output() {
    cout << bestScore << endl;
    cout << bestSeedRow << " " << bestSeedCol << endl;
    for (int ind = 0; ind < bestScore; ++ind) {
        cout << bestColors[ind] << endl;
    }
}

void init() {
    // initialize best solution
    bestScore = -1;
    // initialize random generator
    srand(123456);
}

void read_standard_input() {
    cin >> n >> c;
    for (int row = 0; row < n; ++row) {
        for (int col = 0; col < n; ++col) {
            cin >> mat[row][col];
        }
    }
}

void dfs(int row, int col, int v) {
    used[row][col] = 1;
    vertex[row][col] = v;
    for (int i = 0; i < 4; ++i) {
        row += mx[i];
        col += my[i];

        if (row >= 0 && row < n && col >= 0 && col <= n) {
            if (!used[row][col] && mat[row][col] == mat[row-mx[i]][col-my[i]]) {
                dfs(row, col, v);
            }
        }

        row -= mx[i];
        col -= my[i];
    }
}

void solve_initiated1(int vv) {
    score = 0;
    seedRow = vertexRow[vv];
    seedCol = vertexCol[vv];
    memset(status, 0, sizeof status);
    status[vv] = 1;
    int done = 1;
    memset(nReady, 0, sizeof nReady);
    memset(nValue, 0, sizeof nValue);
    for (int col = 1; col <= c; ++col) {
        value[col].reset();
    }
    for (int idx = 0; idx < nA[vv]; ++idx) {
        status[A[vv][idx]] = 2;
    }
    for (int idx = 0; idx < nA[vv]; ++idx) {
        int u = A[vv][idx];
        ready[color[u]][nReady[color[u]]] = u;
        ++nReady[color[u]];
        for (int idx1 = 0; idx1 < nA[u]; ++idx1) {
            int x = A[u][idx1];
            if (status[x] == 0) {
                if (!value[color[u]][x]) {
                    ++nValue[color[u]];
                    value[color[u]].set(x);
                }
            }
        }
    }
    while (done < n_vertices) {
        if ((double)(clock() - started)/CLOCKS_PER_SEC >= 0.8) return;
        int best = -1;
        int bsz = -1;
        for (int ind = 1; ind <= c; ++ind) {
            int col = cc[ind];
            int sz = nValue[col];
            if (nReady[col] > 0) ++sz;
            if (sz > 0) {
                if (best == -1 || bsz < sz) {
                    best = col;
                    bsz = sz;
                }
            }
        }

        if (best == -1) {
            for (int col = 1; col <= c; ++col) if (nReady[col] > 0) {
                colors[score] = col;
                ++score;
            }
            break;
        }

        colors[score] = best;
        ++score;
        for (int ind = 0; ind < nReady[best]; ++ind) {
            int v = ready[best][ind];
            ++done;
            status[v] = 1;
            for (int idx = 0; idx < nA[v]; ++idx) {
                int u = A[v][idx];
                if (status[u] == 0) {
                    ready[color[u]][nReady[color[u]]] = u;
                    ++nReady[color[u]];
                    status[u] = 2;
                    for (int idx1 = 0; idx1 < nA[u]; ++idx1) {
                        int x = A[u][idx1];
                        if (status[x] == 0) {
                            if (!value[color[u]][x]) {
                                ++nValue[color[u]];
                                value[color[u]].set(x);
                            }
                        }
                        if (value[color[x]][u]) {
                            --nValue[color[x]];
                            value[color[x]].reset(u);
                        }
                    }
                }
            }
        }
        nReady[best] = 0;
    }

    if (bestScore == -1 || score < bestScore) {
        bestScore = score;
        bestSeedRow = seedRow;
        bestSeedCol = seedCol;
        for (int i = 0; i < score; ++i) {
            bestColors[i] = colors[i];
        }
    }
}

bool my_comp(const int& i, const int& j) {
    if (nA[i] > nA[j]) return true;
    return false;
}

void solve1() {
    memset(used, 0, sizeof used);
    n_vertices = 0;
    for (int row = 0; row < n; ++row) {
        for (int col = 0; col < n; ++col) if (!used[row][col]) {
            vertexRow[n_vertices] = row;
            vertexCol[n_vertices] = col;
            color[n_vertices] = mat[row][col];
            dfs(row, col, n_vertices);
            ++n_vertices;
        }
    }
    for (int i = 0; i < n_vertices; ++i) {
        adj[i].clear();
    }
    for (int row = 0; row < n; ++row) {
        for (int col = 0; col < n; ++col) {
            for (int i = 0; i < 4; ++i) {
                if (row + mx[i] < 0 || row + mx[i] >= n ||
                        col + my[i] < 0 || col + my[i] >= n) {
                    continue;
                }
                if (vertex[row][col] != vertex[row+mx[i]][col+my[i]]) {
                    adj[vertex[row][col]].insert(vertex[row+mx[i]][col+my[i]]);
//                    adj[vertex[row+mx[i]][col+my[i]]].insert(vertex[row][col]);
                }
            }
        }
    }
    for (int i = 0; i < n_vertices; ++i) {
        nA[i] = (int)adj[i].size();
        A[i] = new int[nA[i]];
        int j = 0;
        for (set<int>::iterator it = adj[i].begin(); it != adj[i].end(); ++it) {
            A[i][j++] = *it;
        }
    }
//    int best_start = -1;
//    int best = -1;
//    for (int i = 0; i < n_vertices; ++i) {
//        if (best == -1 || (int)adj[i].size() > best) {
//            best = adj[i].size();
//            best_start = i;
//        }
//    }
//    solve_initiated1(best_start);
    vector<int> ind(n_vertices);
    for (int i = 0; i < n_vertices; ++i) ind[i] = i;
    sort(ind.begin(), ind.end(), my_comp);
    for (int i = 0; i <= c; ++i) cc[i] = i;

    while (true) {
        for (int i = 0; i < n_vertices; ++i) {
            if ((double)(clock() - started) / CLOCKS_PER_SEC >= 0.8) return;
            solve_initiated1(ind[i]);
        }
        random_shuffle(cc + 1, cc + c);
    }
}

int main() {
    started = clock();
    read_input();
//    read_standard_input();
    init();
    solve1();
    output_best_solution();
//    output_standard_output();
    return 0;
}
